export PATH="/sbin:/bin:/usr/sbin:/usr/bin:$PATH"

# NOTE: do not prepend !/bin/sh this script is executed directly via execve()

FEATURES="RTC_WRITE INITTAB_PREPARE EXEC_RESUME CLEAN_OLD_PACKAGES"
NOREBOOT_FROM=""
PRE_INST_PKGS="firmware-zwave-ctrlr"
# check if these packages are still depended upon and if not remove them if they are installed (should be done automatically)
CHECK_QMF_PACKAGES_REMOVED="hcb-upnp hcb-core hcb-web flash-gui flash-qwidget-diy flash-lite-adobe"


SCRIPTPATH=`realpath $0`
SCRIPTDIR=`dirname $SCRIPTPATH`
SCRIPTNAME=`basename $SCRIPTPATH`
SCRIPTSTATUS=$0.res

export HCB_ARCH="$1"
export HCB_FLAV="$2"
export HCB_VERS="$3"
export ACTION="$4"

featureEnabled()
{
	# test if this feature is enabled
	echo "$FEATURES" | grep -q "$1"
}

dropboxMessage()
{
	SUBJECT="$1"
	SUBTITLE="$2"
	MESSAGE="$3"
	ACTIONS="$4"
	
	echo ">> dropboxMessage $MESSAGE"
	
	time=`date +%s`
	Message="
	<Config>
	<message>
	 <content>$MESSAGE</content>
	  <subject>$SUBJECT</subject>
	  <subtitle>$SUBTITLE</subtitle>
	  <messageType>critical</messageType>
	  <received_t>$time</received_t>
	  <internalAddress>dropboxed_$time</internalAddress>
	  <read_t></read_t>
	  <readMsgSend>1</readMsgSend>
	  <hideOnRemove>0</hideOnRemove>
	  <package>happ_usermsg</package>
	  <type>message</type>
	  <name>upgrade_msg</name>
	  <visibility>1</visibility>$ACTIONS
	 </message>
	</Config>"
	
	echo "$Message" > /tmp/dropbox_msg.xml
	mv /tmp/dropbox_msg.xml /HCBv2/config/dropbox/
}

statusSet()
{
	STATUS="$1"
	echo "$STATUS" > $SCRIPTSTATUS
	echo ">> `date +'%Y-%m-%d %H:%M:%S %z'` (pid $$) $STATUS"
}

errorExit()
{
	STATUS="$ACTION-error-$1"
	STRING="$2"
	echo "$STATUS" > $SCRIPTSTATUS
	echo ">> `date +'%Y-%m-%d %H:%M:%S %z'` (pid $$) $STATUS '$STRING'"
	exit 1
}

fbInitTextMode()
{
	# dim down fast
	echo '0:2' > /sys/class/backlight/ed2.0-bl.0/ramp_brightness_sync
	# setting mode reinits the console (and with it wipes the background)
	cat /sys/class/graphics/fb0/modes > /sys/class/graphics/fb0/mode
	# unblank
	echo 0 > /sys/class/graphics/fb0/blank
	# turn off blanking timeouts for terminal
	echo -n -e '\33[9;0]\33[14;0]'> /dev/tty0
	# clear screen (also done since boot-logos@46)
	echo -n -e '\33[2J'> /dev/tty0
	# reshow boot image
	/etc/init.d/show-fb-image start start
	# start output at line 23
	echo -n -e '\33[23;0f'> /dev/tty0
}

fbLinePrint()
{
	echo ">>> fbLinePrint: $1"
	echo "   $1" > /dev/tty0
}

fbLineUpdateStatus()
{
	DOTS="   ..."
	while sleep 1
	do
		PERC="`sed /tmp/update.status.vars -n -r -e 's,^.+item=(.+?)&items=(.+?)&.+$,\1,p' 2>/dev/null`"
		PERC="${PERC:-0}%"

		# do not append newline, \r to beginning of line after print, append space to overwrite prev-longer-sentences
		echo -n -e "   Voortgang: $PERC ${DOTS:0:3}    \r" > /dev/tty0

		# shift right
		DOTS="${DOTS:5:1}${DOTS:0:5}"
	done
}

# removes all packages which have a newer version downloaded
removeOldPackages()
{
	featureEnabled CLEAN_OLD_PACKAGES || return 0
	
	PREV_DIR=$PWD
	cd $PKGCACHE || return 1
	echo "removing old packages..."
	
	# find unique strings before the first '_'
	UNIQUE_PACKAGES=`ls *_*_${HCB_ARCH}.ipk | cut -d _ -f 1 | sort -u`
	
	PACKAGES_TO_REMOVE=""
	for p in $UNIQUE_PACKAGES
	do
		# list reversed, sorted by created date on a single line
		PACKAGE_VERSIONS=`ls ${p}_*_$HCB_ARCH.ipk -cr1`
		# get count, head -n -1 doesn't work with busybox' head
		OLD_VERSION_COUNT=$((`echo "$PACKAGE_VERSIONS" | wc -l` - 1))
		# remove latest version from remove list
		OLD_VERSIONS=`echo "$PACKAGE_VERSIONS" | head -n $OLD_VERSION_COUNT -`
		
		[ -n "$OLD_VERSIONS" ] && PACKAGES_TO_REMOVE="$PACKAGES_TO_REMOVE `echo $OLD_VERSIONS | xargs`"
	done
	
	if [ -n "$PACKAGES_TO_REMOVE" ]
	then
		echo "Removing the following packages because a newer version is downloaded: $PACKAGES_TO_REMOVE"
		rm $PACKAGES_TO_REMOVE
	fi
	
	cd $PREV_DIR
	return 0
}

# run after S39hostname.sh, before S41networking
EXEC_RESUME_SCRIPT=/etc/rcS.d/S40upgrade-execute-resume.sh

execResumeInProgress()
{
	if featureEnabled EXEC_RESUME
	then
		if [ "$ACTION" = "execute-resume" ] #[ -f $EXEC_RESUME_SCRIPT ]
		then
			return 0 # true
		fi
	fi
	return 1 # false
}

execSanityChecks()
{
	#* opkg_conf_load: Creating temp dir /tmp/opkg-0uAYDE failed: No such file or directory.
	if [ ! -w /tmp/ ]
	then
		fbLinePrint "> Repareren /tmp/"
		mount /proc 2>/dev/null
		grep -q /var/volatile /proc/mounts || mount /var/volatile
		mkdir -p /var/volatile/tmp
		[ ! -w /tmp/ ] && fbLinePrint "> Repareren /tmp/ [mislukt]"
	fi
}

# create a file that will resume the upgrade-execute at next boot
execResumeScheduleCheckForNextBoot()
{
	if featureEnabled EXEC_RESUME
	then
		if execResumeInProgress
		then
			# show progress on console
		  tail -f $SCRIPTPATH.log </dev/null >/dev/console 2>&1 &
		  
			echo ">> Detected execute-resume, notifying the user via framebuffer."

			fbInitTextMode
			fbLinePrint "Afgebroken software update naar versie ${HCB_VERS} gedetecteerd."
			fbLinePrint "Update wordt hervat, even geduld a.u.b... zet uw Toon NIET uit tijdens de update."
			
			execSanityChecks
			
			# this means that the file is still here since the last (aborted) upgrade,
			# so we actually remove it to prevent an endless upgrade-loop!
			#echo ">> Detected execute-resume script from previous run, removing it."
			rm -f $EXEC_RESUME_SCRIPT
			
			# start a process that watches /tmp/update.status.vars
			fbLineUpdateStatus &
			STATUS_UPDATE_PID=$!
			echo ">> Spawned /tmp/update.status.vars watcher with PID $STATUS_UPDATE_PID"

			# we are running during early boot, HCBv2 apps are not running yet
			STOPSTARTAPPS=0
		else
			echo ">> Creating $EXEC_RESUME_SCRIPT with args '${HCB_ARCH} ${HCB_FLAV} ${HCB_VERS} execute-resume'"
	
			echo "#!/bin/sh" > $EXEC_RESUME_SCRIPT
			echo "echo '$0: Aborted upgrade-execute for ${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS} detected,'" >> $EXEC_RESUME_SCRIPT
			echo "echo '$0: executing $SCRIPTPATH execute-resume'" >> $EXEC_RESUME_SCRIPT
			echo "$SCRIPTPATH ${HCB_ARCH} ${HCB_FLAV} ${HCB_VERS} execute-resume" >> $EXEC_RESUME_SCRIPT
			echo "echo '$0: done.'" >> $EXEC_RESUME_SCRIPT
			chmod a+x $EXEC_RESUME_SCRIPT
		fi
	
		sync
	fi
}

execResumeRemoveCheckForNextBoot()
{
	if featureEnabled EXEC_RESUME
	then
		if execResumeInProgress
		then
			# stop the process that watches /tmp/update.status.vars
			echo ">> Killing /tmp/update.status.vars watcher with PID $STATUS_UPDATE_PID"
			kill -9 $STATUS_UPDATE_PID
			
			if [ "$REBOOT" = "1" ]
			then
				fbLinePrint "Software update afgerond, Toon start nu opnieuw op..."
			else
				fbLinePrint "Software update afgerond, Toon start nu..."
			fi
			
			# give people time to read the message
			sleep 3
		fi
		
		echo ">> Removing $EXEC_RESUME_SCRIPT"
		rm -f $EXEC_RESUME_SCRIPT
	fi
}

# migrate the files in /HCBv2/tmp/opkg-cache to /mnt/data/update if hcb_config supports the new location
migratePkgCacheDirectory()
{
	OLD_PKGCACHE='/HCBv2/tmp/opkg-cache'
	NEW_PKGCACHE='/mnt/data/update'
	if [ -d "$OLD_PKGCACHE" ] && strings /HCBv2/sbin/hcb_config | grep -q -e "^${NEW_PKGCACHE}\$"
	then
		echo ">> migrating files to '${NEW_PKGCACHE}' ..."
		if [ ! -d "$NEW_PKGCACHE" ]
		then
			# Normally that should not happen since the directory should be created by /etc/init.d/populate-volatile.sh
			echo "the directory '$NEW_PKGCACHE' is missing."
			mkdir -p "$NEW_PKGCACHE" || errorExit "opkg-migrate" "Failed to create the directory '$NEW_PKGCACHE'"
			echo "the directory '$NEW_PKGCACHE' was created."
		fi

		# If the dir is empty, don't do anything otherwise 'mv' will fail
		if [ "$(ls -A "$OLD_PKGCACHE")" ]
		then
			mv "$OLD_PKGCACHE"/* "$NEW_PKGCACHE"/ || echo "failed to move files to '$NEW_PKGCACHE'"
		else
			echo 'nothing to migrate'
		fi
		rmdir "$OLD_PKGCACHE" || echo "failed to remove the old directory '$OLD_PKGCACHE'."
		sync
		echo ">> migrating files to '${NEW_PKGCACHE}': finished"
	fi
}

(
	echo ""
	echo ">> `date +'%Y-%m-%d %H:%M:%S %z'` (pid $$) $SCRIPTNAME $@"
	echo ""
	#echo "$$" > $SCRIPTPATH.pid
	statusSet "$ACTION"
	cd $SCRIPTDIR
	
	PKGCACHE=$SCRIPTDIR
	BASE_FEED_URL="http://feed.hae.int/feeds"
	CN="`hostname`"
	HCB_AFV="${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS}"
	REBOOT=1
	STOPSTARTAPPS=1
	CLEANUP="`grep -q '^root:DISABLED' /etc/passwd && echo 1`"
	HCBV2_UP="`netstat -ltn | grep -q '0.0.0.0:1337' && echo 1`"
	
	# Migration of files from the old to new location is performed the very first time
	# after the hcb_config supports the new location.
	migratePkgCacheDirectory

	case $ACTION in
	
		#####################################################################################################
		prepare)
		
			statusSet "preparing"
			
			# clean up some space for downloading new packages
			removeOldPackages
			
			# Hotfix for writing System Time to RTC (ET-794)
			if featureEnabled RTC_WRITE
			then
				echo ">> Hotfix for writing System Time to RTC"
				echo "Hardware Clock before: `hwclock --show`"
				# here we know the system time is ok: script downloaded -> internet -> hae-timesync
				hwclock --systohc --utc
				echo "Hardware Clock after : `hwclock --show`"
			fi

			# set extra pkg system feeds
			EXTRA_FEEDS="`wget -q "${BASE_FEED_URL}/${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS}/SystemFeed" -O -`"
			if [ "$EXTRA_FEEDS" = "" ]
			then
				echo "Failed retrieving '${BASE_FEED_URL}/${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS}/SystemFeed', trying again verbosely:"
				wget "${BASE_FEED_URL}/${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS}/SystemFeed" -O - 2>&1
				errorExit "dl-afv" "AFV download failed Maybe sync to Nas? Maybe non-existing?"
			fi
		
			# set correct feed location for initial install
			rm -f /etc/opkg/*-feed.conf /usr/lib/opkg/lists/*
			echo "src/gz base ${BASE_FEED_URL}/${HCB_ARCH}/${HCB_FLAV}/${HCB_VERS}" > /etc/opkg/base-feed.conf
			echo "$EXTRA_FEEDS" | {
				while read EF
				do
					EF_NAME="`echo "$EF" | cut -d ' ' -f1`"
					EF_PATH="`echo "$EF" | cut -d ' ' -f2`"
					echo "src/gz ${EF_NAME} ${BASE_FEED_URL}/${HCB_ARCH}/${EF_PATH}" > /etc/opkg/${EF_NAME}-feed.conf
				done;
			}
			
			echo ">> configured opkg feeds:"
			cat /etc/opkg/*-feed.conf
			
			echo ">> opkg update"
			opkg update 2>&1; EXITCODE=$?
			
			[ "$EXITCODE" != "0" ] && errorExit "opkg-update" "Exit code $EXITCODE for: opkg update"
			
			echo ">> opkg --cache $PKGCACHE upgrade --download-only"
			opkg --cache $PKGCACHE upgrade --download-only 2>&1; EXITCODE=$?

			# intermediate cleanup
			rm -rf /tmp/opkg-*
			# ensure the log greps below always work
			sync 
			
			if [ "$EXITCODE" != "0" ]
			then
				# * opkg_install_pkg: Package xxx md5sum mismatch. Either the opkg or the package index are corrupt. 
				if tail -n20 $SCRIPTPATH.log | grep -q 'md5sum mismatch'
				then
					rm -f $PKGCACHE/*.ipk
					errorExit "opkg-upgrade-md5sum" "opkg upgrade found an md5sum mismatch: removed *.ipk!"
				fi
				
				# * opkg_download: Failed to download http://...ipk: Timeout was reached.				
				# * opkg_download: Failed to download http://...ipk: Couldn't connect to server.
				if tail -n20 $SCRIPTPATH.log | grep -q 'Failed to download'
				then
					errorExit "opkg-upgrade-download" "opkg upgrade download failed"
				fi
				
				# generic error
				errorExit "opkg-upgrade" "Exit code $EXITCODE for: opkg upgrade"
			fi
			
			echo ">> downloaded:"
			
			# remove packages of which we just downloaded a newer version
			removeOldPackages
			
			du -ch $PKGCACHE/*.ipk | grep total

			ACTIONS=""
			SUBJECT="Software update"
			SUBTITLE="Softwareversie $HCB_VERS is beschikbaar."
			MESSAGE="Softwareversie $HCB_VERS voor uw ToonRR is nu beschikbaar. Ga naar Systeem -> Software om de software van Toon bij te werken.BRBRDit is een automatisch gegenereerd bericht."
			
			if [ -e /HCBv2/sbin/qt-gui ]
			then
				MESSAGE='Er is nieuwe software beschikbaar voor uw Toon. Druk hieronder op de knop Instellingen en daarna op Update en tot slot op Bijwerken om de software van Toon bij te werken.BRBRDit is een automatisch gegenereerd bericht.'
				ACTIONS="
	  <actions>
	   <action>
		<type>button</type>
		<btnLabel>Instellingen</btnLabel>
		<btnPos>left</btnPos>
		<cmdName>maximize</cmdName>
		<cmdArg>System.Software</cmdArg>
		<cmdTarget>widget-settings</cmdTarget>
		<returnAfterPress>false</returnAfterPress>
	   </action>
	  </actions>"
			else
				SUPPORTS_MESSAGES="`strings /HCBv2/swf/widget-note-nl_NL.swf | grep -q 'cmdTarget' && echo 1`"
				if [ "$SUPPORTS_MESSAGES" = "1" ]
				then
					#check if we are going to a Qt version
					HCB_VERS_MAJOR=`echo $HCB_VERS | cut -f1 -d.`
					if [ $HCB_VERS_MAJOR -ge 3 ] 
					then
						MESSAGE='Er is nieuwe software beschikbaar voor uw Toon. Druk hieronder op de knop Systeem en daarna op Update en tot slot op Bijwerken om de software van Toon bij te werken. Deze update duurt wat langer dan normaal omdat er veel wordt vernieuwd. Toon wordt sneller, persoonlijker en gebruiksvriendelijker.BRBRDit is een automatisch gegenereerd bericht.'
					else
						MESSAGE='Er is nieuwe software beschikbaar voor uw ToonRR. Druk hieronder op de knop "Systeem" en vervolgens op "Update" om de software van Toon bij te werken. BRBRDit is een automatisch gegenereerd bericht.'
					fi
					ACTIONS="
	  <actions>
	   <action>
	    <type>button</type>
	    <btnLabel>Systeem</btnLabel>
	    <btnPos>left</btnPos>
	    <cmdName>maximize</cmdName>
	    <cmdArg>System.Software</cmdArg>
	    <cmdTarget>widget-settings</cmdTarget>
	    <returnAfterPress>false</returnAfterPress>
	   </action>
	  </actions>"
				fi
			fi
			
			dropboxMessage "$SUBJECT" "$SUBTITLE" "$MESSAGE" "$ACTIONS"
			
			statusSet "prepared"
			
			;;

		#####################################################################################################
		execute|execute-resume)

			statusSet "executing"

			#OLD_VERSION="`cat /etc/opkg/base-feed.conf | cut -d'/' -f 8 | cut -d'.' -f1-2`"
			# note base-feed.conf is already overwritten here!
			OLD_MAJOR_MINOR="`opkg list_installed 'base-*-*' | sed -r -e 's,^.+ - ([0-9]+\.[0-9]+)\.[0-9]+-.+$,\\1,'`"
			OLD_MAJOR_MINOR_MICRO="`opkg list_installed 'base-*-*' | sed -r -e 's,^.+ - ([0-9]+\.[0-9]+\.[0-9]+)-.+$,\\1,'`"
			
			# check if we can suffice with a "soft reboot"
			if featureEnabled NOREBOOT_FROM
			then
				# (grep --word-regexp: Select only those lines containing matches that form whole words.)
				# check for word-match in list of versions
				if echo "$NOREBOOT_FROM" | grep -q -w "$OLD_MAJOR_MINOR_MICRO"
				then
					echo ">> Current version $OLD_MAJOR_MINOR_MICRO in ($NOREBOOT_FROM), doing soft reboot."
					REBOOT=0
				else
					echo ">> Current version $OLD_MAJOR_MINOR_MICRO not in ($NOREBOOT_FROM), doing regular reboot."
				fi
			fi
			
			if [ "$HCBV2_UP" ]
			then
				MSG="Upgrading application software from version '$OLD_MAJOR_MINOR_MICRO' to '$HCB_AFV'."
				echo ">> Logging \"$MSG\" to 99_DEBUG.log.json"
				/HCBv2/bin/bxt \
					-d $CN:hcb_log \
					-s Logger \
					-n LogMessage \
					-a facility -v 1 \
					-a categoryId -v 99 \
					-a timestampGenerated -v "`date +'%Y-%m-%d %H:%M:%S %z'`" \
					-a logText -v "$MSG" \
					-w 0 >/dev/null 2>/dev/null &
					
				sleep 1
			
				echo ">> Sending InitiatePreExit to hcb_config."
				/HCBv2/bin/bxt \
					-d $CN:hcb_config \
					-n InitiatePreExit \
					-w 0 >/dev/null 2>/dev/null &
			fi

			execResumeScheduleCheckForNextBoot
			
			if [ "$STOPSTARTAPPS" = "1" ]
			then
				echo ">> Sleep 2, give time to return 'upgrade started' messages to UI"
				sleep 2

				echo ">> Stopping HCBv2 apps + prevent restart..."
				/etc/init.d/HCBv2 stop
			fi

			# prevent restarting during upgrade
			echo 'exit' > /tmp/etc-default-HCBv2
			chmod a+x /tmp/etc-default-HCBv2

			if [ "$STOPSTARTAPPS" = "1" ]
			then
				# kill this one especially, left running in inittab by design.
				killall -9 hcb_netcon
				# NOTE: flash keeps running during upgrade, restart by inittab would fail because of /tmp/etc-default-HCBv2
				
				echo ">> Disabling watchdog /dev/watchdog"
				rm -f /dev/watchdog
				mknod /dev/watchdog c 10 130
				# make sure we don't reboot:
				echo V > /dev/watchdog
				rm -f /dev/watchdog
			fi
			
			if [ "$PRE_INST_PKGS" != "" ]
			then
				#opkg info firmware-zwave-ctrlr | grep -q "firmware-zwave-ctrlr" && echo OK
				echo ">> opkg --cache $PKGCACHE install $PRE_INST_PKGS"
				opkg --cache $PKGCACHE install $PRE_INST_PKGS 2>&1;
			fi
			
			# opkg sometimes (incorrectly?) returns 255, therefore do 3 retries
			for RETRY_ON_EXITCODE in 1 2 3
			do
				echo ">> opkg --cache $PKGCACHE upgrade - attempt $RETRY_ON_EXITCODE"
				opkg --cache $PKGCACHE upgrade 2>&1; EXITCODE=$?
				
				echo ">> exitcode for opkg --cache $PKGCACHE upgrade was $EXITCODE"
				
				if [ "$EXITCODE" = "0" ]
				then	
					break
				fi
			done

			# did any of the packages just installed request a reboot?
			if [ -f /tmp/opkg_reboot ]
			then
				echo ">> Detected /tmp/opkg_reboot, will reboot when done."
				REBOOT=1
			fi

			execResumeRemoveCheckForNextBoot

			# intermediate cleanup
			rm -rf /tmp/opkg-*
			rm -rf /var/lib/opkg/*
			sync
			
			if [ "$EXITCODE" != "0" ]
			then
				# 
				# TODO: check if we can use /tmp/update.status.vars to communicate error
				#       to user in a more verbose manner instead of fb img "upfail"?
				#
			
				# stop flash/qt anim
				killall -9 flash qt-gui
				# show upgrade error msg
				/etc/init.d/show-fb-image upfail
				# and bail out...
				errorExit "opkg-upgrade" "Exit code $EXITCODE for: opkg upgrade"
			fi

			echo ">> Checking if these packages are not installed: $CHECK_QMF_PACKAGES_REMOVED"
			for a in $CHECK_QMF_PACKAGES_REMOVED
			do
				echo "checking $a"
				if ! opkg depends base-$HCB_ARCH-$HCB_FLAV | grep -q $a && [ "`opkg list_installed $a`" != "" ]
				then
					echo "removing $a as it is no longer depended upon by base package and it isn't removed."
					opkg remove --force-depends $a
				fi
			done

			# upgrade OK: remove tmpfiles
			echo ">> upgrade OK: removing package cache files..."
			rm -f $PKGCACHE/*.ipk
			
			if [ "$CLEANUP" = "1" ]
			then
				rm -f $PKGCACHE/_opkg.*.script.*
				rm -f /etc/opkg/*-feed.conf
			fi

			# prepare a message for the user to read later ## following four lines commented out by Jakob on 24-11-2015 10:45 for AET-709
			#SUBJECT="Software update"
			#SUBTITLE="Software update naar $HCB_VERS geslaagd."
			#MESSAGE="De software update is geslaagd. De software van uw ToonRR is bijgewerkt naar versie $HCB_VERS.BRBRDit is een automatisch gegenereerd bericht."
			#dropboxMessage "$SUBJECT" "$SUBTITLE" "$MESSAGE"

			sync
			
			if [ "$REBOOT" = "1" ]
			then
				# prepare the contents of inittab for next boot, only writes if changed
				if featureEnabled INITTAB_PREPARE
				then
					rm -f /tmp/etc-default-HCBv2
					/etc/init.d/HCBv2 prepare-start
					sync
				fi

				# TODO: trigger watchdog here?
				
				echo ">> Done, scheduling reboot in 2 secs..."
				{ sleep 2 && shutdown -r now >/dev/null 2>&1; } >/dev/null 2>&1 &

			else # no reboot

				# remove the "exit" in this file, so flash+HCBv2 apps can start:
				rm -f /tmp/etc-default-HCBv2

				# now we restart them?
				if [ "$STOPSTARTAPPS" = "1" ]
				then
					echo ">> Restarting flash..."
					killall -9 flash qt-gui
					# dims down, give time to quickly load flash + dim up
					sleep 3
					echo ">> Starting HCBv2 apps..."
					/etc/init.d/HCBv2 start
				fi
			
				echo ">> Done."
			fi

			# prevent last _opkg.*.script.sh.res file from being left behind
			if [ "$CLEANUP" != "1" ]
			then
				statusSet "executed"
			fi
		
			;;


		#####################################################################################################
		*)
			echo ">> Unknown ACTION arg! Aborting."
			errorExit "unknown-action" "Unknown ACTION arg! Aborting."
			;;

	esac
	
	echo ">> `date +'%Y-%m-%d %H:%M:%S %z'` (pid $$) exit-0 $SCRIPTNAME $@"

) </dev/null >>$SCRIPTPATH.log 2>&1

